#!/bin/sh

# set -e

Live ()
{
#       N="^[[0;39m"    # Normal color
        N="\033[0;39m"    # Normal color
#       R="^[[1;31m"    # Failure message
        R="\033[1;31m"    # Failure message
#       G="^[[1;32m"    # Success message
        G="\033[1;34m"    # Success message
#       Y="^[[1;33m"    # Variable Descriptions
        Y="\033[1;36m"    # Variable Descriptions
#       W="^[[1;37m"    # Brackets
        W="\033[1;37m"    # Brackets
        A="\033[1;33m"    # Amarillo
        V="\033[1;32m"    # Verde

	if [ -x /scripts/local-top/cryptroot ]
	then
		/scripts/local-top/cryptroot
	fi

	exec 6>&1
	exec 7>&2
	exec > boot.log
	exec 2>&1
	tail -f boot.log >&7 &
	tailpid="${!}"

    clear
    printf "\n${W}[${Y} Starting Elive Boot System ${W}]${N}\n\n"

	LIVE_BOOT_CMDLINE="${LIVE_BOOT_CMDLINE:-$(cat /proc/cmdline)}"
	Cmdline_old

    if [ "$FLL_CHANGEDEVICE" = "true" ] ; then
            printf "${V}Please insert your USB with Elive now,\n"
            printf "${V}We will continue the boot in 30 seconds${N}\n\n"
        umount /dev/cdrom 2>/dev/null
        umount /dev/sr0 2>/dev/null
        umount -l /dev/cdrom 2>/dev/null
        umount -l /dev/sr0 2>/dev/null
        eject 2>/dev/null
        eject /dev/cdrom 2>/dev/null
        eject /dev/sr0 2>/dev/null # macbook
            #read nada # do not enable, it bugs
            sleep 30
    fi

    #
    # Print kernel info
    #
    printf "${G}Linux Kernel\t${W}[ ${Y}$(uname -r)${W} ]${N}\n"

    #
    # Print CPU info
    #
    fll_cpuinfo

    #
    # Print System Memory info
    #
    eval $(fll_meminfo)
    printf "${G}System Memory\t${W}"
    printf "[ ${Y}${MEMUSED}M used ${W}] "
    printf "[ ${Y}${MEMFREE}M free${W} ] "
    printf "[ ${Y}${MEMTOTAL}M total${W} ]"
    printf "${N}\n\n"

	Debug

	Read_only

	Select_eth_device

	if [ -e /conf/param.conf ]
	then
		. /conf/param.conf
	fi

	# Needed here too because some things (*cough* udev *cough*)
	# changes the timeout

	if [ ! -z "${NETBOOT}" ] || [ ! -z "${FETCH}" ] || [ ! -z "${HTTPFS}" ] || [ ! -z "${FTPFS}" ]
	then
		if do_netmount
		then
			livefs_root="${mountpoint}"
		else
			panic "Unable to find a live file system on the network"
		fi
	else
		if [ -n "${ISCSI_PORTAL}" ]
		then
			do_iscsi && livefs_root="${mountpoint}"
		elif [ -n "${PLAIN_ROOT}" ] && [ -n "${ROOT}" ]
		then
			# Do a local boot from hd
			livefs_root=${ROOT}
		else
			if [ -x /usr/bin/memdiskfind ]
			then
				MEMDISK=$(/usr/bin/memdiskfind)

				if [ $? -eq 0 ]
				then
					# We found a memdisk, set up phram
					# Sometimes "modprobe phram" can not successfully create /dev/mtd0.
				        # Have to try several times.
					max_try=20
					while [ ! -c /dev/mtd0 -a "$max_try" -gt 0 ]; do
					  modprobe phram phram=memdisk,${MEMDISK}
					  sleep 0.2
					  if [ -c /dev/mtd0 ]; then
					  	break
					  else
					  	rmmod phram
				  	  fi
					  max_try=$((max_try - 1))
					done

					# Load mtdblock, the memdisk will be /dev/mtdblock0
					modprobe mtdblock
				fi
			fi

			# Scan local devices for the image
			i=0
			while [ "$i" -lt 140 ]
			do
				livefs_root=$(find_livefs ${i})

				if [ -n "${livefs_root}" ]
				then
					break
				fi

				sleep 1
				i="$(($i + 1))"
			done
		fi
	fi

	if [ -z "${livefs_root}" ]
	then
		panic "Unable to find a medium containing a live file system"
	fi

	Verify_checksums "${livefs_root}"

	if [ "${TORAM}" ]
	then
		live_dest="ram"
	elif [ "${TODISK}" ]
	then
		live_dest="${TODISK}"
	fi

	if [ "${live_dest}" ]
	then
		log_begin_msg "Copying live media to ${live_dest}"
		copy_live_to "${livefs_root}" "${live_dest}"
		log_end_msg
	fi

	# if we do not unmount the ISO we can't run "fsck /dev/ice" later on
	# because the mountpoint is left behind in /proc/mounts, so let's get
	# rid of it when running from RAM
	if [ -n "$FROMISO" ] && [ "${TORAM}" ]
	then
		losetup -d /dev/loop0

		if is_mountpoint /run/live/fromiso
		then
			umount /run/live/fromiso
			rmdir --ignore-fail-on-non-empty /run/live/fromiso \
				>/dev/null 2>&1 || true
		fi
	fi

	if [ -n "${MODULETORAMFILE}" ] || [ -n "${PLAIN_ROOT}" ]
	then
		setup_unionfs "${livefs_root}" "${rootmnt}"
	else
		mac="$(get_mac)"
		mac="$(echo ${mac} | sed 's/-//g')"
		mount_images_in_directory "${livefs_root}" "${rootmnt}" "${mac}"
	fi

	if [ -n "${ROOT_PID}" ]
	then
		echo "${ROOT_PID}" > "${rootmnt}"/lib/live/root.pid
	fi

	log_end_msg

	# aufs2 in kernel versions around 2.6.33 has a regression:
	# directories can't be accessed when read for the first the time,
	# causing a failure for example when accessing /var/lib/fai
	# when booting FAI, this simple workaround solves it
	ls /root/* >/dev/null 2>&1

	# if we do not unmount the ISO we can't run "fsck /dev/ice" later on
	# because the mountpoint is left behind in /proc/mounts, so let's get
	# rid of it when running from RAM
	if [ -n "$FINDISO" ] && [ "${TORAM}" ]
	then
		losetup -d /dev/loop0

		if is_mountpoint /run/live/findiso
		then
			umount /run/live/findiso
			rmdir --ignore-fail-on-non-empty /run/live/findiso \
				>/dev/null 2>&1 || true
		fi
	fi

	if [ -L /root/etc/resolv.conf ] ; then
		# assume we have resolvconf
		DNSFILE="${rootmnt}/etc/resolvconf/resolv.conf.d/base"
	else
		DNSFILE="${rootmnt}/etc/resolv.conf"
	fi
	if [ -f /etc/resolv.conf ] && ! grep -E -q -v '^[[:space:]]*#|^[[:space:]]*$' ${DNSFILE}
	then
		log_begin_msg "Copying /etc/resolv.conf to ${DNSFILE}"
		cp -v /etc/resolv.conf ${DNSFILE}
		log_end_msg
	fi

	if ! [ -d "/lib/live/boot" ]
	then
		panic "A wrong rootfs was mounted."
	fi

	# avoid breaking existing user scripts that rely on the old path
	# this includes code that checks what is mounted on /lib/live/mount/*
	# (eg: grep /lib/live /proc/mount)
	# XXX: to be removed before the bullseye release
	mkdir -p ${rootmnt}/lib/live/mount
	mount --rbind /run/live ${rootmnt}/lib/live/mount

	Fstab
	Netbase

	Swap

    Remove_things

    if [ "$FLL_LOGIN" = "true" ] ; then
        . /scripts/functions
        . /lib/live/boot/9990-initramfs-tools.sh
        panic "user requested a login shell"
    fi

	exec 1>&6 6>&-
	exec 2>&7 7>&-
	kill ${tailpid}
	[ -w "${rootmnt}/var/log/" ] && mkdir -p "${rootmnt}/var/log/live" && ( \
				cp boot.log "${rootmnt}/var/log/live" 2>/dev/null; \
				cp fsck.log "${rootmnt}/var/log/live" 2>/dev/null )
}
